module CU( clk, reset, opcode, ac8, clrPC, we, selB, selA, readM, f, ldAC, ldIR, 
           ldAR, incPC, ldPC, selAR, ps );
			  
input clk, ac8, reset;
input [3:0] opcode;
output clrPC, we, selB, selA, readM, ldAC, ldIR, ldAR, incPC, ldPC, selAR;
output [2:0] f;
output [4:0] ps;
reg [13:0] CV;
reg [4:0] y, Y;

parameter [4:0] S0=0, S1=1, S2=2, S3=3, S4=4, S5=5, S6=6, S7=7, S8=8, S9=9,
					 S10=10, S11=11, S12=12, S13=13, S14=14, S15=15, S16=16, S17=17;
					 
parameter [3:0] ADR=4'b0000, MOV=4'b0001, SHR=4'b0010, SHL=4'b0011, ADD=4'b0100,
					 SUB=4'b0101, XOR=4'b0110, AND=4'b0111, MVI=4'b1000, MVD=4'b1001,
					 LDA=4'b1010, STA=4'b1011, BRA=4'b1100, JMP=4'b1101, HALT=4'b1110,
					 NOP=4'b1111;
					 
always @ (negedge reset or negedge clk)
	if (!reset) y<=S0;
	else y<=Y;
	
always @ (opcode or y or ac8)
		case (y)
			S0: Y=S1;
			S1: Y=S2;
			S2: if (opcode==NOP) Y=S17;
			    else if (opcode[3:2]==00) Y=S3;
				else Y=S4;			
			S3: Y=S1;
			S4: if (opcode[2:0]==000) Y=S8;
				else Y=S5;
			S5: if (opcode[3]==0) Y=S6;
				else case (opcode)
					MVD: Y=S9;
					LDA: Y=S11;
					STA: Y=S13;
					BRA: Y=S14;
					JMP: Y=S15;
					HALT: Y=S16;
					default Y=S0;
				endcase
			S6: Y=S7;
			S7: Y=S1;
			S8: Y=S1;
			S9: Y=S10;
			S10: Y=S1;
			S11: Y=S12;
			S12: Y=S1;
			S13: Y=S1;
			S14: if (ac8) Y=S15; else Y=S1;
			S15: Y=S1;
			S16: Y=S16;
			S17: Y=S1;
			default: Y=S0;
		endcase
	
always @ ( y or opcode )
	case (y)
		
		S0: CV=14'h0000;
		S1: CV=14'h2000;
		S2: CV=14'h2014;
		S3: case ( opcode )
				ADR: CV=14'h3800;
				MOV: CV=14'h3900;
				SHR: CV=14'h39C0;
				SHL: CV=14'h3980;
				default: CV=14'hxxxx;
			 endcase
		S4: CV=14'h2004;
		S5: CV=14'h2009;
		S6: case ( opcode )
				ADD: CV=14'h2401;
				SUB: CV=14'h2441;
				XOR: CV=14'h24C1;
				AND: CV=14'h2481;
				default: CV=14'hxxxx;
			endcase
		S7: case ( opcode )
				ADD: CV=14'h2421;
				SUB: CV=14'h2461;
				XOR: CV=14'h24E1;
				AND: CV=14'h24A1;
				default: CV=14'hxxxx;
			endcase
		S8: CV=14'h3100;
		S9: CV=14'h2101;
		S10: CV=14'h3101;
		S11: CV=14'h2101;
		S12: CV=14'h2121;
		S13: CV=14'h2741;
		S14: CV=14'h2000;
		S15: CV=14'h2003;
		S16: CV=14'h2000;
		S17: CV=14'h2000;
		default: CV=14'hxxxx;
	endcase
	
assign selAR=CV[0];
assign ldPC=CV[1];
assign incPC=CV[2];
assign ldAR=CV[3];
assign ldIR=CV[4];
assign ldAC=CV[5];
assign f=CV[8:6];
assign readM=CV[9];
assign selA=CV[10];
assign selB=CV[11];
assign we=CV[12];
assign clrPC=CV[13];
assign ps=y;

endmodule
